from unittest import TestCase
from lr1110evk.SerialExchange import (
    WifiMode,
    WifiEnableMode,
    GnssOption,
    GnssCaptureMode,
    GnssAntennaSelection,
    GnssConstellation,
    CommandStartWifiScan,
    CommandStartWifiCountryCode,
    CommandStartGnssAutonomous,
    CommandStartGnssAssisted,
)
from lr1110evk.BaseTypes import WifiChannels


class CommandConfigureTestCase(TestCase):
    def test_channels_bit_mask_generation(self):
        channel_names = ["CHANNEL_1", "CHANNEL_6", "CHANNEL_11"]
        channels = [WifiChannels.get_channel_from_name(name) for name in channel_names]
        bit_mask = CommandStartWifiScan.channel_list_to_bit_mask(channels)
        self.assertEqual(bit_mask, b"\x21\x04")

    def test_types_bit_mask_generation(self):
        type_names = ["TYPE_B"]
        bit_mask = CommandStartWifiScan.type_list_to_bit_mask(type_names)
        self.assertEqual(bit_mask, b"\x01")

    def test_command_configure_and_start_wifi_scan(self):
        command = CommandStartWifiScan()
        channel_names = ["CHANNEL_1", "CHANNEL_6", "CHANNEL_11"]
        channels = [WifiChannels.get_channel_from_name(name) for name in channel_names]
        command.wifi_channels = channels
        command.wifi_types = ["TYPE_B"]
        command.wifi_nbr_retrials = 255
        command.wifi_max_results_per_scan = 10
        command.wifi_timeout = 64249
        command.wifi_abort_on_timeout = False
        command.wifi_mode = WifiMode.beacon_and_packet
        actual_byte_frame = command.convert_to_bytes()
        expected_byte_frame = b"\x01\x00\x0a\x00\x01\x21\x04\x01\xff\x0A\xf9\xfa\x02\x00"
        self.assertEqual(actual_byte_frame, expected_byte_frame)

    def test_command_configure_and_start_wifi_country_code(self):
        command = CommandStartWifiCountryCode()
        channel_names = ["CHANNEL_1", "CHANNEL_6", "CHANNEL_11"]
        channels = [WifiChannels.get_channel_from_name(name) for name in channel_names]
        command.wifi_channels = channels
        command.wifi_nbr_retrials = 255
        command.wifi_max_results_per_scan = 10
        command.wifi_timeout = 64249
        command.wifi_abort_on_timeout = True
        actual_byte_frame = command.convert_to_bytes()
        expected_byte_frame = b"\x01\x00\x08\x00\x02\x21\x04\xff\x0A\xf9\xfa\x01"
        self.assertEqual(actual_byte_frame, expected_byte_frame)

    def test_command_configure_and_start_gnss_autonomous(self):
        command = CommandStartGnssAutonomous()
        command.gnss_option = GnssOption.best_effort
        command.gnss_capture_mode = GnssCaptureMode.dual
        command.gnss_nb_satellite = 10
        command.gnss_antenna_selection = GnssAntennaSelection.select_antenna_1
        command.gnss_constellation_mask = GnssConstellation.gps.value
        actual_byte_frame = command.convert_to_bytes()
        expected_byte_frame = b"\x01\x00\x06\x00\x03\x01\x01\x0A\x01\x01"
        self.assertEqual(actual_byte_frame, expected_byte_frame)

    def test_command_configure_and_start_gnss_assisted(self):
        command = CommandStartGnssAssisted()
        command.gnss_option = GnssOption.best_effort
        command.gnss_capture_mode = GnssCaptureMode.dual
        command.gnss_nb_satellite = 10
        command.gnss_antenna_selection = GnssAntennaSelection.select_antenna_1
        command.gnss_constellation_mask = GnssConstellation.gps.value
        actual_byte_frame = command.convert_to_bytes()
        expected_byte_frame = b"\x01\x00\x06\x00\x04\x01\x01\x0A\x01\x01"
        self.assertEqual(actual_byte_frame, expected_byte_frame)
