from unittest import TestCase, skip
from lr1110evk.FieldTestPost.Core.FileReader import FileReader
from lr1110evk.BaseTypes import WifiChannels, ScannedGnss
import datetime


class TestFileReader(TestCase):
    def test_wifi_line_reading(self):
        expected_mac_address = "68:a3:78:94:c6:d9"
        expected_channel_name = "CHANNEL_6"
        str_line = "[2019-06-24 17:18:06.479049] [2 - 1] 68:a3:78:94:c6:d9, CHANNEL_6, TYPE_B, -72, 12934, 5010, 25091, 0"
        result_line = FileReader.get_scan_info_and_job_data_from_line(str_line)

        expected_wifi_channel = WifiChannels.get_channel_from_name(
            expected_channel_name
        )
        self.assertEqual(result_line.scan_info.mac_address, expected_mac_address)
        self.assertEqual(result_line.scan_info.wifi_channel, expected_wifi_channel)

    def test_gnss_line_reading(self):
        str_line = "[2019-10-21 12:42:58.751156] [3 - 0] 0101070808e210a8ef20e22e0014529bec007d0080a4522becb70e20513d41e5e9dd45b05a2115da3b8b67957d73c8f498caaa216e9aec23, 0, 250, 1150"
        result_line = FileReader.get_scan_info_and_job_data_from_line(str_line)
        self.assertEqual(
            result_line.scan_info.nav_message,
            "0101070808e210a8ef20e22e0014529bec007d0080a4522becb70e20513d41e5e9dd45b05a2115da3b8b67957d73c8f498caaa216e9aec23",
        )
        self.assertEqual(result_line.scan_info.radio_timing_ms, "250")
        self.assertEqual(result_line.scan_info.computation_timing_ms, "1150")

    def test_gnss_from_line(self):
        str_line = "0101070808e210a8ef20e22e0014529bec007d0080a4522becb70e20513d41e5e9dd45b05a2115da3b8b67957d73c8f498caaa216e9aec23, 0, 250, 1150"
        fake_scan_time = datetime.datetime.utcnow()
        scanned_object = ScannedGnss.from_csv_line(fake_scan_time, str_line)
        self.assertEqual(
            scanned_object.nav_message,
            "0101070808e210a8ef20e22e0014529bec007d0080a4522becb70e20513d41e5e9dd45b05a2115da3b8b67957d73c8f498caaa216e9aec23",
        )
        self.assertEqual(scanned_object.radio_timing_ms, "250")
        self.assertEqual(scanned_object.computation_timing_ms, "1150")
