from unittest import TestCase
from lr1110evk.FieldTestPost.Core.Geocoding import GeocodingBuilder, GeoCodingData


class GeoCodingAssert(TestCase):
    def assertGeoCodingEqual(self, expected: GeoCodingData, actual: GeoCodingData):
        if not isinstance(expected, GeoCodingData):
            raise AssertionError(
                "Wrong argument type: expected {}, received {}".format(
                    type(expected), type(actual)
                )
            )
        if not isinstance(actual, GeoCodingData):
            raise AssertionError(
                "Wrong argument type: expected {}, received {}".format(
                    type(expected), type(actual)
                )
            )

        self.assertEqual(expected.street, actual.street, "Streets are different")
        self.assertEqual(expected.town, actual.town, "Town are different")
        self.assertEqual(expected.country, actual.country, "Country are different")
        self.assertEqual(
            expected.country_code, actual.country_code, "Country code are different"
        )
        self.assertEqual(expected.state, actual.state, "State are different")
        self.assertEqual(expected.postcode, actual.postcode, "Postcode are different")
        self.assertEqual(expected.house, actual.house, "House are different")


class GeoCodingBuilderTestCase(GeoCodingAssert, TestCase):
    def test_build_europe(self):
        simple_reverse_geocoding = {
            "country": "France",
            "country_code": "fr",
            "county": "Isère",
            "municipality": "La Tour-du-Pin",
            "postcode": "38730",
            "road": "Chemin du Village",
            "state": "Auvergne-Rhône-Alpes",
            "village": "Chassignieu",
        }
        expected_geo_coding_data = GeoCodingData(
            street="Chemin du Village",
            town="La Tour-du-Pin",
            country="France",
            country_code="fr",
            state="Auvergne-Rhone-Alpes",
            postcode="38730",
            house=None,
        )

        geo_conding_builder = GeocodingBuilder()
        geo_conding_builder.load_names_aliases()
        reverse_geocoding = geo_conding_builder.build_data(simple_reverse_geocoding)

        self.assertGeoCodingEqual(expected_geo_coding_data, reverse_geocoding)

    def test_build_asia(self):
        # Data obtained with https://osm-geo.loraclouddemo.com/osm?latitude=31.18&longitude=121.40
        simple_reverse_geocoding = {
            "city": "Xuhui District",
            "country": "China",
            "country_code": "cn",
            "county": "Xuhui District",
            "postcode": "200233",
            "road": "Guiqing Road",
            "state": "Shanghai",
        }
        expected_geo_coding_data = GeoCodingData(
            street="Guiqing Road",
            town="Xuhui District",
            country="China",
            country_code="cn",
            state="Shanghai",
            postcode="200233",
            house=None,
        )

        geo_conding_builder = GeocodingBuilder()
        geo_conding_builder.load_names_aliases()
        reverse_geocoding = geo_conding_builder.build_data(simple_reverse_geocoding)

        self.assertGeoCodingEqual(expected_geo_coding_data, reverse_geocoding)

    def test_build_asia_non_latin_characters(self):
        # Data obtained with https://osm-geo.loraclouddemo.com/osm?latitude=31.18&longitude=121.50
        simple_reverse_geocoding = {
            "city": "Pudong",
            "country": "China",
            "country_code": "cn",
            "postcode": "200126",
            "residential": "上南花苑（#2, #3）",
            "road": "Yuntai Road",
            "state": "Shanghai",
        }
        expected_geo_coding_data = GeoCodingData(
            street="Yuntai Road",
            town="Pudong",
            country="China",
            country_code="cn",
            state="Shanghai",
            postcode="200126",
            house=None,
        )

        geo_conding_builder = GeocodingBuilder()
        geo_conding_builder.load_names_aliases()
        reverse_geocoding = geo_conding_builder.build_data(simple_reverse_geocoding)

        self.assertGeoCodingEqual(expected_geo_coding_data, reverse_geocoding)
