from unittest import TestCase
from lr1110evk.Tools.KmlGenerator import KmlGenerator
from lr1110evk.Job.KmlExport import kmlOutput


class KmlGeneratorTestCase(TestCase):
    def test_line_wifi_parsing(self):
        line = {
            "uplink": {
                "devEui": "2202900000000032",
                "port": 199,
                "f_counter": 17,
                "rx_timestamp": "2020-09-11T07:16:19.086Z",
                "frequency_hz": 867300000,
                "spreading_factor": 10,
                "bandwith_hz": 125000,
                "airtime": 411648000,
                "lns": {
                    "name": "ttn-v2",
                    "context": {
                        "app_id": "test-modem-evk-eu",
                        "dev_id": "modem-evk-eui",
                        "hardware_serial": "2202900000000032",
                        "port": 199,
                        "counter": 17,
                        "payload_raw": "FAXhAORKSlWgRR3O",
                        "payload_fields": {"error": "0x14 is not implemented"},
                        "metadata": {
                            "time": "2020-09-11T07:16:19.08684244Z",
                            "frequency": 867.3,
                            "modulation": "LORA",
                            "data_rate": "SF10BW125",
                            "airtime": 411648000,
                            "coding_rate": "4/5",
                            "gateways": [
                                {
                                    "gtw_id": "eui-7276fffffe0198ac",
                                    "timestamp": 1389272012,
                                    "time": "2020-09-11T07:16:17.956852Z",
                                    "channel": 1,
                                    "rssi": -130,
                                    "snr": -14,
                                    "rf_chain": 0,
                                    "latitude": 45.17334,
                                    "longitude": 5.71246,
                                    "altitude": 256,
                                },
                                {
                                    "gtw_id": "eui-7276fffffe019881",
                                    "timestamp": 2867662484,
                                    "time": "2020-09-11T07:16:17.956852Z",
                                    "channel": 1,
                                    "rssi": -120,
                                    "snr": -3,
                                    "rf_chain": 0,
                                    "latitude": 45.18779,
                                    "longitude": 5.68838,
                                    "altitude": 242,
                                },
                                {
                                    "gtw_id": "eui-7276fffffe019843",
                                    "timestamp": 3594734740,
                                    "time": "2020-09-11T07:16:17.956842Z",
                                    "channel": 1,
                                    "rssi": -119,
                                    "snr": -2,
                                    "rf_chain": 0,
                                    "latitude": 45.18553,
                                    "longitude": 5.71976,
                                    "altitude": 246,
                                },
                                {
                                    "gtw_id": "eui-904d4afffeff4c03",
                                    "timestamp": 1185123148,
                                    "time": "2020-09-11T07:16:17.95685Z",
                                    "antenna": 1,
                                    "channel": 17,
                                    "rssi": -116,
                                    "snr": -1,
                                    "rf_chain": 0,
                                    "latitude": 45.19073,
                                    "longitude": 5.73685,
                                    "altitude": 248,
                                },
                                {
                                    "gtw_id": "eui-7276fffffe01988f",
                                    "timestamp": 972920412,
                                    "time": "2020-09-11T07:16:17.95685Z",
                                    "channel": 1,
                                    "rssi": -115,
                                    "snr": 0,
                                    "rf_chain": 0,
                                    "latitude": 45.19044,
                                    "longitude": 5.70349,
                                    "altitude": 254,
                                },
                                {
                                    "gtw_id": "eui-7276fffffe019882",
                                    "timestamp": 2119960748,
                                    "time": "2020-09-11T07:16:17.956848Z",
                                    "channel": 1,
                                    "rssi": -111,
                                    "snr": 5,
                                    "rf_chain": 0,
                                    "latitude": 45.17956,
                                    "longitude": 5.70991,
                                    "altitude": 246,
                                },
                            ],
                        },
                    },
                },
                "payload_hex": "1405e100e44a4a55a0451dce",
                "payload_bytes": {
                    "type": "Buffer",
                    "data": [20, 5, 225, 0, 228, 74, 74, 85, 160, 69, 29, 206],
                },
            },
            "das": '{"result": {"22-02-90-00-00-00-00-32": {"result": {"deveui": "22-02-90-00-00-00-00-32", "usage": {"das": 11, "gnss": 0, "wifi": 5, "total": 16, "period": {"from": 1599782400.0, "intvl": 86400}}, "pending_requests": {"requests": [], "id": 7, "updelay": 32, "upcount": 0}, "info_fields": {"rfu": null, "temp": {"value": 28, "timestamp": 1599754211.175}, "charge": {"value": 0, "timestamp": 1599754211.175}, "deveui": null, "region": null, "rxtime": {"value": 0, "timestamp": 1599754211.175}, "signal": {"value": {"snr": 0.5, "rssi": -26}, "timestamp": 1599754211.175}, "status": {"value": {"mute": false, "crash": false, "joined": true, "upload": false, "suspend": false, "brownout": false}, "timestamp": 1599754211.175}, "uptime": {"value": 0, "timestamp": 1599754211.175}, "adrmode": null, "alcsync": {"value": {"time": 60, "token": 0}, "timestamp": 1599658438.098}, "chipeui": null, "joineui": null, "session": {"value": 1095, "timestamp": 1599754211.175}, "voltage": null, "crashlog": null, "firmware": {"value": {"fwcrc": "35e2a1c1", "fwtotal": 65535, "fwcompleted": 0}, "timestamp": 1599754211.175}, "interval": null, "rstcount": {"value": 1544, "timestamp": 1599754211.175}, "appstatus": null, "streampar": null}, "log_messages": [], "fports": {"dmport": 199, "gnssport": 198, "wifiport": 197, "streamport": 199}, "dnlink": null, "fulfilled_requests": [], "file": null, "stream_records": null, "position_solution": {"algorithm_type": "wifi", "llh": [45.189455, 5.717595, 0], "accuracy": 13, "timestamp": 1599808579.4979856}}}}}',
            "record": "AFTER_DAS_LOC",
        }
        generator = KmlGenerator("toto", "titi")
        kml_points = generator.interpret_line(line)
        self.assertEqual(
            1, len(kml_points), msg="The line interpreted contains only one result"
        )
        self.assertEqual("2020-09-11T07:16:19.086Z", kml_points[0].date_str)
        self.assertEqual(kmlOutput.SCAN_TYPE_WIFI, kml_points[0].kml_output_type)
        self.assertEqual(45.189455, kml_points[0].coordinate.latitude)
        self.assertEqual(5.717595, kml_points[0].coordinate.longitude)
        self.assertEqual(0, kml_points[0].coordinate.altitude)
        self.assertIsNone(kml_points[0].metadata)

    def test_line_gnss_parsing(self):
        line = {
            "uplink": {
                "devEui": "2202900000000032",
                "port": 199,
                "f_counter": 31,
                "rx_timestamp": "2020-09-11T16:43:46.961Z",
                "frequency_hz": 867700000,
                "spreading_factor": 10,
                "bandwith_hz": 125000,
                "airtime": 575488000,
                "lns": {
                    "name": "ttn-v2",
                    "context": {
                        "app_id": "test-modem-evk-eu",
                        "dev_id": "modem-evk-eui",
                        "hardware_serial": "2202900000000032",
                        "port": 199,
                        "counter": 31,
                        "payload_raw": "FJk9AQVZfQesamfNeBwHW9VmiYLaDYiDMzsEAKDaeiteAAA=",
                        "payload_fields": {"error": "0x14 is not implemented"},
                        "metadata": {
                            "time": "2020-09-11T16:43:46.96140555Z",
                            "frequency": 867.7,
                            "modulation": "LORA",
                            "data_rate": "SF10BW125",
                            "airtime": 575488000,
                            "coding_rate": "4/5",
                            "gateways": [
                                {
                                    "gtw_id": "eui-904d4afffeff4dd3",
                                    "timestamp": 3436847340,
                                    "time": "2020-09-11T16:43:45.818737Z",
                                    "channel": 3,
                                    "rssi": -128,
                                    "snr": -15,
                                    "rf_chain": 0,
                                    "latitude": 45.17093,
                                    "longitude": 5.74144,
                                    "altitude": 251,
                                },
                                {
                                    "gtw_id": "eui-904d4afffeff4c03",
                                    "timestamp": 873202940,
                                    "time": "2020-09-11T16:43:45.818704Z",
                                    "antenna": 1,
                                    "channel": 19,
                                    "rssi": -117,
                                    "snr": -7,
                                    "rf_chain": 0,
                                    "latitude": 45.19071,
                                    "longitude": 5.73686,
                                    "altitude": 246,
                                },
                                {
                                    "gtw_id": "eui-7276fffffe0198ac",
                                    "timestamp": 1077380940,
                                    "time": "2020-09-11T16:43:45.818703Z",
                                    "channel": 3,
                                    "rssi": -121,
                                    "snr": -4,
                                    "rf_chain": 0,
                                    "latitude": 45.17336,
                                    "longitude": 5.71245,
                                    "altitude": 260,
                                },
                                {
                                    "gtw_id": "eui-7276fffffe019881",
                                    "timestamp": 2555783740,
                                    "time": "2020-09-11T16:43:45.818702Z",
                                    "channel": 3,
                                    "rssi": -121,
                                    "snr": -3,
                                    "rf_chain": 0,
                                    "latitude": 45.18778,
                                    "longitude": 5.68837,
                                    "altitude": 235,
                                },
                                {
                                    "gtw_id": "eui-7276fffffe019843",
                                    "timestamp": 3282842140,
                                    "time": "2020-09-11T16:43:45.818693Z",
                                    "channel": 3,
                                    "rssi": -118,
                                    "snr": -2,
                                    "rf_chain": 0,
                                    "latitude": 45.18551,
                                    "longitude": 5.71977,
                                    "altitude": 259,
                                },
                                {
                                    "gtw_id": "eui-7276fffffe019882",
                                    "timestamp": 1808075908,
                                    "time": "2020-09-11T16:43:45.818697Z",
                                    "channel": 3,
                                    "rssi": -111,
                                    "snr": 5,
                                    "rf_chain": 0,
                                    "latitude": 45.1796,
                                    "longitude": 5.70988,
                                    "altitude": 252,
                                },
                                {
                                    "gtw_id": "eui-7276fffffe01988f",
                                    "timestamp": 661032676,
                                    "time": "2020-09-11T16:43:45.818701Z",
                                    "channel": 3,
                                    "rssi": -108,
                                    "snr": 7,
                                    "rf_chain": 0,
                                    "latitude": 45.19044,
                                    "longitude": 5.7035,
                                    "altitude": 257,
                                },
                            ],
                        },
                    },
                },
                "payload_hex": "14993d0105597d07ac6a67cd781c075bd5668982da0d8883333b0400a0da7a2b5e0000",
                "payload_bytes": {
                    "type": "Buffer",
                    "data": [
                        20,
                        153,
                        61,
                        1,
                        5,
                        89,
                        125,
                        7,
                        172,
                        106,
                        103,
                        205,
                        120,
                        28,
                        7,
                        91,
                        213,
                        102,
                        137,
                        130,
                        218,
                        13,
                        136,
                        131,
                        51,
                        59,
                        4,
                        0,
                        160,
                        218,
                        122,
                        43,
                        94,
                        0,
                        0,
                    ],
                },
            },
            "das": '{"result": {"22-02-90-00-00-00-00-32": {"result": {"deveui": "22-02-90-00-00-00-00-32", "usage": {"das": 26, "gnss": 1, "wifi": 5, "total": 31, "period": {"from": 1599782400.0, "intvl": 86400}}, "pending_requests": {"requests": [], "id": 7, "updelay": 32, "upcount": 0}, "info_fields": {"rfu": null, "temp": {"value": 26, "timestamp": 1599840835.586}, "charge": {"value": 0, "timestamp": 1599840835.586}, "deveui": null, "region": null, "rxtime": {"value": 9, "timestamp": 1599840835.586}, "signal": {"value": {"snr": 0.0, "rssi": -102}, "timestamp": 1599840835.586}, "status": {"value": {"mute": false, "crash": false, "joined": true, "upload": false, "suspend": false, "brownout": false}, "timestamp": 1599840835.586}, "uptime": {"value": 9, "timestamp": 1599840835.586}, "adrmode": null, "alcsync": {"value": {"time": 60, "token": 0}, "timestamp": 1599658438.098}, "chipeui": null, "joineui": null, "session": {"value": 1095, "timestamp": 1599754211.175}, "voltage": null, "crashlog": null, "firmware": {"value": {"fwcrc": "35e2a1c1", "fwtotal": 65535, "fwcompleted": 0}, "timestamp": 1599754211.175}, "interval": null, "rstcount": {"value": 1544, "timestamp": 1599754211.175}, "appstatus": null, "streampar": null}, "log_messages": [], "fports": {"dmport": 199, "gnssport": 198, "wifiport": 197, "streamport": 199}, "dnlink": null, "fulfilled_requests": [], "file": null, "stream_records": null, "position_solution": {"ecef": [4480512.44, 448592.53, 4502415.88], "llh": [45.18925, 5.71745, 310.34], "capture_time_gps": 1283877811.309347, "gdop": 3.33, "accuracy": 16.0, "capture_time_utc": 1599842593.309347, "timestamp": 1599842627.4947226}}}}}',
            "record": "AFTER_DAS_LOC",
        }
        generator = KmlGenerator("toto", "titi")
        kml_points = generator.interpret_line(line)
        self.assertEqual(
            1, len(kml_points), msg="The line interpreted contains only one result"
        )
        self.assertEqual("2020-09-11T16:43:46.961Z", kml_points[0].date_str)
        self.assertEqual(kmlOutput.SCAN_TYPE_GNSS, kml_points[0].kml_output_type)
        self.assertEqual(45.18925, kml_points[0].coordinate.latitude)
        self.assertEqual(5.71745, kml_points[0].coordinate.longitude)
        self.assertEqual(310.34, kml_points[0].coordinate.altitude)
        self.assertIsNone(kml_points[0].metadata)
